@@ -4,7 +4,7 @@
 # can be used by non-unix systems that do not have built-in support
 # for shells.
 #
-# RCS: @(#) $Id: console.tcl,v 1.8.2.4 2001/10/19 19:40:17 das Exp $
+# RCS: @(#) $Id: console.tcl,v 1.8.2.6 2002/06/22 08:52:24 hobbs Exp $
 #
 # Copyright (c) 1998-1999 Scriptics Corp.
 # Copyright (c) 1995-1997 Sun Microsystems, Inc.
@@ -27,9 +27,9 @@ namespace eval ::tk::console {
     variable defaultPrompt  ; # default prompt if tcl_prompt1 isn't used
 
     if {$inPlugin} {
-	set defaultPrompt {subst "[history nextid] % "}
+	set defaultPrompt {subst {[history nextid] % }}
     } else {
-	set defaultPrompt {subst "([file tail [pwd]]) [history nextid] % "}
+	set defaultPrompt {subst {([file tail [pwd]]) [history nextid] % }}
     }
 }
 
@@ -285,8 +285,10 @@ proc tkConsoleBind {w} {
 
     ## Get all Text bindings into Console
     foreach ev [bind Text] { bind Console $ev [bind Text $ev] }	
-    ## We really didn't want the newline insertion
+    ## We really didn't want the newline insertion...
     bind Console <Control-Key-o> {}
+    ## ...or any Control-v binding (would block <<Paste>>)
+    bind Console <Control-Key-v> {}
 
     # For the moment, transpose isn't enabled until the console
     # gets and overhaul of how it handles input -- hobbs
@@ -307,6 +309,7 @@ proc tkConsoleBind {w} {
 	<<Console_PrevSearch>>		<Control-Key-r>
 	<<Console_NextSearch>>		<Control-Key-s>
 
+	<<Console_Expand>>		<Key-Tab>
 	<<Console_Expand>>		<Key-Escape>
 	<<Console_ExpandFile>>		<Control-Shift-Key-F>
 	<<Console_ExpandProc>>		<Control-Shift-Key-P>
@@ -326,11 +329,6 @@ proc tkConsoleBind {w} {
 	bind Console $key {}
     }
 
-    bind Console <Tab> {
-	tkConsoleInsert %W \t
-	focus %W
-	break
-    }
     bind Console <<Console_Expand>> {
 	if {[%W compare insert > promptEnd]} {::tk::console::Expand %W}
     }
