@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacDraw.c,v 1.6 1999/08/10 05:04:53 jingham Exp $
+ * RCS: @(#) $Id: tkMacDraw.c,v 1.8.2.1 2002/04/08 09:00:32 das Exp $
  */
 
 #include "tkInt.h"
@@ -370,6 +370,9 @@ TkPutImage(
 	pixmap.pmTable = NULL;
 	pixmap.pmReserved = 0;
 	pixmap.baseAddr = image->data;
+    if (image->bytes_per_line >= 0x4000) {
+    	panic("TkImage too wide!");
+    }
 	pixmap.rowBytes = image->bytes_per_line | 0x8000;
 	
 	CopyBits((BitMap *) &pixmap, &((GrafPtr) destPort)->portBits, 
@@ -975,6 +978,26 @@ TkMacSetUpGraphicsPort(
     if (gc->line_width > 1) {
 	PenSize(gc->line_width, gc->line_width);
     }
+    if (gc->line_style != LineSolid) {
+	unsigned char *p = (unsigned char *) &(gc->dashes);
+	/*
+	 * Here the dash pattern should be set in the drawing,
+	 * environment, but I don't know how to do that for the Mac.
+	 *
+	 * p[] is an array of unsigned chars containing the dash list.
+	 * A '\0' indicates the end of this list.
+	 *
+	 * Someone knows how to implement this? If you have a more
+	 * complete implementation of SetUpGraphicsPort() for
+	 * the Mac (or for Windows), please let me know.
+	 *
+	 *	Jan Nijtmans
+	 *	CMG Arnhem, B.V.
+	 *	email: j.nijtmans@chello.nl (private)
+	 *	       jan.nijtmans@cmg.nl (work)
+	 *	url:   http://purl.oclc.org/net/nijtmans/
+	 */
+    }
 }
 
 /*
@@ -1129,7 +1152,7 @@ InvertByte(
     }
     return result;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1171,4 +1194,3 @@ TkpDrawHighlightBorder (
         }
     }
 }
-        
\ No newline at end of file
