@@ -9,11 +9,11 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacMenus.c,v 1.2 1998/09/14 18:23:38 stanton Exp $
+ * RCS: @(#) $Id: tkMacMenus.c,v 1.6.2.1 2001/04/04 21:39:49 hobbs Exp $
  */
 
 #include "tcl.h"
-#include "tclMacInt.h"
+#include "tclMacInt.h" /* Needed for FSpLocationFromPath */
 #include "tk.h"
 #include "tkInt.h"
 #include "tkMacInt.h"
@@ -80,6 +80,7 @@ TkMacHandleMenuSelect(
     Str255 name;
     Tk_Window tkwin;
     Window window;
+    TkDisplay *dispPtr;
 
     if (mResult == 0) {
     	TkMacHandleTearoffMenu();
@@ -105,7 +106,7 @@ TkMacHandleMenuSelect(
 			break;
 		    }
 		default:
-		    GetItem(tkAppleMenu, theItem, name);
+		    GetMenuItemText(tkAppleMenu, theItem, name);
 		    HiliteMenu(0);
 		    OpenDeskAcc(name);
 		    return;
@@ -119,8 +120,13 @@ TkMacHandleMenuSelect(
 		    break;
 		case kCloseItem:
 		    /* Send close event */
-		    window = TkMacGetXWindow(FrontWindow());
-		    tkwin = Tk_IdToWindow(tkDisplayList->display, window);
+		    if (TkMacHaveAppearance() >= 0x110) {
+		        window = TkMacGetXWindow(FrontNonFloatingWindow());
+		    } else {
+		        window = TkMacGetXWindow(FrontWindow());
+		    }
+		    dispPtr = TkGetDisplayList();
+		    tkwin = Tk_IdToWindow(dispPtr->display, window);
 		    TkGenWMDestroyEvent(tkwin);
 		    break;
 		case kQuitItem:
@@ -189,7 +195,7 @@ TkMacInitMenus(
     InsertMenu(tkAppleMenu, 0);
     AppendMenu(tkAppleMenu, "\pAbout Tcl & Tk");
     AppendMenu(tkAppleMenu, "\p(-");
-    AddResMenu(tkAppleMenu, 'DRVR');
+    AppendResMenu(tkAppleMenu, 'DRVR');
 
     if (TkMacUseMenuID(kFileMenu) != TCL_OK) {
     	panic("Menu ID %d is already in use!", kFileMenu);
@@ -251,9 +257,15 @@ GenerateEditEvent(
     Point where;
     Tk_Window tkwin;
     Window window;
+    TkDisplay *dispPtr;
 
-    window = TkMacGetXWindow(FrontWindow());
-    tkwin = Tk_IdToWindow(tkDisplayList->display, window);
+    if (TkMacHaveAppearance() >= 0x110) {
+        window = TkMacGetXWindow(FrontNonFloatingWindow());
+    } else {
+        window = TkMacGetXWindow(FrontWindow());
+    }
+    dispPtr = TkGetDisplayList();
+    tkwin = Tk_IdToWindow(dispPtr->display, window);
     tkwin = (Tk_Window) ((TkWindow *) tkwin)->dispPtr->focusPtr;
     if (tkwin == NULL) {
 	return;
@@ -317,29 +329,26 @@ GenerateEditEvent(
 static void 
 SourceDialog()
 {
-    StandardFileReply reply;
-    OSType fileTypes[1];
-    OSErr err;
-    int length, result;
-    Handle path;
+    int result;
+    char *path;
+    char openCmd[] = "tk_getOpenFile -filetypes {\
+            {{TCL Scripts} {.tcl} TEXT} {{Text Files} {} TEXT}}";
     
     if (gInterp == NULL) {
 	return;
     }
     
-    fileTypes[0] = 'TEXT';
-    StandardGetFile(NULL, 1, fileTypes, &reply);
-    if (reply.sfGood == false) {
+    if (Tcl_Eval(gInterp, openCmd) != TCL_OK) {
 	return;
     }
     
-    err = FSpPathFromLocation(&reply.sfFile, &length, &path);
-    if (err == noErr) {
-	HLock(path);
-	result = Tcl_EvalFile(gInterp, *path);
-	HUnlock(path);
-	DisposeHandle(path);
+    path = Tcl_GetStringResult(gInterp);
+    
+    if (strlen(path) == 0) {
+        return;
     }
+    
+    result = Tcl_EvalFile(gInterp, path);
     if (result == TCL_ERROR) {
 	Tcl_BackgroundError(gInterp);
     }	   
