@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) xcolors.c 1.3 96/12/17 13:07:02
+ * RCS: @(#) $Id: xcolors.c,v 1.3 2000/03/02 23:53:26 hobbs Exp $
  */
 
 #include <tkInt.h>
@@ -105,12 +105,18 @@ static XColorEntry xColors[] = {
     "cyan2", 0, 238, 238,
     "cyan3", 0, 205, 205,
     "cyan4", 0, 139, 139,
+    "dark blue", 0, 0, 139,
+    "dark cyan", 0, 139, 139,
     "dark goldenrod", 184, 134, 11,
+    "dark gray", 169, 169, 169,
     "dark green", 0, 100, 0,
+    "dark grey", 169, 169, 169,
     "dark khaki", 189, 183, 107,
+    "dark magenta", 139, 0, 139,
     "dark olive green", 85, 107, 47,
     "dark orange", 255, 140, 0,
     "dark orchid", 153, 50, 204,
+    "dark red", 139, 0, 0,
     "dark salmon", 233, 150, 122,
     "dark sea green", 143, 188, 143,
     "dark slate blue", 72, 61, 139,
@@ -118,13 +124,18 @@ static XColorEntry xColors[] = {
     "dark slate grey", 47, 79, 79,
     "dark turquoise", 0, 206, 209,
     "dark violet", 148, 0, 211,
+    "DarkBlue", 0, 0, 139,
+    "DarkCyan", 0, 139, 139,
     "DarkGoldenrod", 184, 134, 11,
     "DarkGoldenrod1", 255, 185, 15,
     "DarkGoldenrod2", 238, 173, 14,
     "DarkGoldenrod3", 205, 149, 12,
     "DarkGoldenrod4", 139, 101, 8,
+    "DarkGray", 169, 169, 169,
     "DarkGreen", 0, 100, 0,
+    "DarkGrey", 169, 169, 169,
     "DarkKhaki", 189, 183, 107,
+    "DarkMagenta", 139, 0, 139,
     "DarkOliveGreen", 85, 107, 47,
     "DarkOliveGreen1", 202, 255, 112,
     "DarkOliveGreen2", 188, 238, 104,
@@ -140,6 +151,7 @@ static XColorEntry xColors[] = {
     "DarkOrchid2", 178, 58, 238,
     "DarkOrchid3", 154, 50, 205,
     "DarkOrchid4", 104, 34, 139,
+    "DarkRed", 139, 0, 0,
     "DarkSalmon", 233, 150, 122,
     "DarkSeaGreen", 143, 188, 143,
     "DarkSeaGreen1", 193, 255, 193,
@@ -458,6 +470,7 @@ static XColorEntry xColors[] = {
     "light goldenrod", 238, 221, 130,
     "light goldenrod yellow", 250, 250, 210,
     "light gray", 211, 211, 211,
+    "light green", 144, 238, 144,
     "light grey", 211, 211, 211,
     "light pink", 255, 182, 193,
     "light salmon", 255, 160, 122,
@@ -486,6 +499,7 @@ static XColorEntry xColors[] = {
     "LightGoldenrod4", 139, 129, 76,
     "LightGoldenrodYellow", 250, 250, 210,
     "LightGray", 211, 211, 211,
+    "LightGreen", 144, 238, 144,
     "LightGrey", 211, 211, 211,
     "LightPink", 255, 182, 193,
     "LightPink1", 255, 174, 185,
@@ -840,9 +854,9 @@ FindColor(name, colorPtr)
     if (l > u) {
 	return 0;
     }
-    colorPtr->red = xColors[i].red << 8;
-    colorPtr->green = xColors[i].green << 8;
-    colorPtr->blue = xColors[i].blue << 8;
+    colorPtr->red = ((xColors[i].red << 8) | xColors[i].red);
+    colorPtr->green = ((xColors[i].green << 8) | xColors[i].green);
+    colorPtr->blue = ((xColors[i].blue << 8) | xColors[i].blue);
     return 1;
 }
 
@@ -882,9 +896,12 @@ XParseColor(display, map, spec, colorPtr)
 	if (sscanf(spec+1, fmt, &red, &green, &blue) != 3) {
 	    return 0;
 	}
-	colorPtr->red = ((unsigned short) red) << (4 * (4 - i));
-	colorPtr->green = ((unsigned short) green) << (4 * (4 - i));
-	colorPtr->blue = ((unsigned short) blue) << (4 * (4 - i));
+	colorPtr->red = (((unsigned short) red) << (4 * (4 - i)))
+	    | ((unsigned short) red);
+	colorPtr->green = (((unsigned short) green) << (4 * (4 - i)))
+	    | ((unsigned short) green);
+	colorPtr->blue = (((unsigned short) blue) << (4 * (4 - i)))
+	    | ((unsigned short) blue);
     } else {
 	if (!FindColor(spec, colorPtr)) {
 	    return 0;
